<?php

Yii::import('ext.signal.PISignalManager');
abstract class PSignalManager extends CApplicationComponent implements PISignalManager
{
    
    //每个模块下搜索的目录
    public $searchDirectories = array(
        'models',
        'components',
    );
    //接收器文件名标志
    public $configFilePattern = '_reciever.php';
    
    protected $_signals;
    protected $_configFiles;
    protected $_recievers;
    
    abstract public function getSignalsKey();
    
    abstract public function getConfigFilesKey();
    
    abstract public function getRecieversKey($name);
	
	public function getConfigFiles()
	{
	    if (!is_array($this->_configFiles)) {
	        $this->_configFiles = $this->load($this->getConfigFilesKey());
    	    if (!is_array($this->_configFiles)) {
    	        return false;
    	    }
	    }
	    return $this->_configFiles;
	}
	
	public function getSignals()
	{
	    if (!is_array($this->_signals)) {
	        $this->_signals = $this->load($this->getSignalsKey());
    	    if (!is_array($this->_signals)) {
    	        return false;
    	    }
	    }
	    return $this->_signals;
	}
	
	public function getSignalRecievers($signal)
	{
	    if (!isset($this->_recievers[$signal])) {
	        $this->_recievers[$signal] = $this->load($this->getRecieversKey($signal));
	    }
	    if (!$this->_recievers[$signal] instanceof CList) {
	        $this->_recievers[$signal] = new CList();
	    }
	    return $this->_recievers[$signal];
	}
	
	public function connect($signal, $reciever)
	{
	    $reciever = $this->createReciever($reciever);
	    $recievers = $this->getSignalRecievers($signal);
	    if (!$recievers->contains($reciever)) {
	        $recievers->add($reciever);
	    }
	}
	
	public function disconnect($signal, $reciever= null)
	{
	    if ($reciever === null) {
	        $this->getSignalRecievers($signal)->clear();
	    } else {
	        $this->getSignalRecievers($signal)->remove($this->createReciever($reciever));
	    }
	}
	
	public function send($signal, $event)
	{
	    $this->dispatch($this->getSignalRecievers($signal), $event);
	}
	
	public function dispatch($recievers, $event)
	{
	    foreach ($recievers as $reciever) {
	        call_user_func(array($reciever['class'], $reciever['method']), $event);
	    }
	}
	
	public function registerSignalRecievers($force = false)
	{
	    if ($force === false && $this->getSignals() !== false) {
	        return;
	    }
	    $this->clearAll();
	    $this->registerSignalRecieversRecursive(Yii::app());
	    
	    foreach ($this->_recievers as $signal => &$recievers) {
	        $recievers = new CList($recievers);
	        $this->save($this->getRecieversKey($signal), $recievers);
	    }
	    $this->save($this->getConfigFilesKey(), $this->_configFiles);
	    $this->save($this->getSignalsKey(), $this->_signals);
	}

	public function registerSignalRecieversRecursive($module)
	{
	    $this->processModuleDirectory($module->getBasePath());
	    foreach (array_keys($module->getModules()) as $id) {
	        $this->registerSignalRecieversRecursive($module->getModule($id));
	    }
	}
	
	public function processModuleDirectory($basePath)
	{
	    foreach ($this->searchDirectories as $directory) {
    	    $fileIterator = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($basePath . DIRECTORY_SEPARATOR . $directory));
    	    foreach ($fileIterator as $fileObj) {
    	        $name = explode($this->configFilePattern, $fileObj->getFileName());
    	        if (isset($name[1]) && $name[1] === '') {
    	            $this->_configFiles[] = $fileObj->getPathName();
    	            $recievers = require($fileObj->getPathName());
    	            foreach ($recievers as $reciever) {
    	                if (!isset($this->_signals[$reciever['signal']])) {
    	                    $this->_signals[$reciever['signal']] = 1;
    	                } else {
    	                    $this->_signals[$reciever['signal']]++;
    	                }
    	            	$this->_recievers[$reciever['signal']][] = $this->createReciever($reciever, $name[0]);
    	            }
    	        }
    	    }
	    }
	}
	
	public function clearAll()
	{
	    $this->_recievers = $this->_configFiles = $this->_signals = null;
	    $this->clearPersister();
	}
	
	public function update()
	{
	    $this->clearAll();
	    $this->registerSignalRecievers(true);
	}
	
	public function createReciever($reciever, $class = '')
	{
	    return array(
            'signal'=>$reciever['signal'],
            'class'=>isset($reciever['class']) ? $reciever['class'] : $class,
            'method'=>$reciever['method'],
	    );
	}
	
	abstract public function load($key);
	abstract public function save($key, $data);
}